/// HttpStatus Class
class HttpStatus {
  final code;
  final message;
  final description;

  const HttpStatus(this.code, this.message, this.description);

  String get formattedString => '$code $message';
}

/// List of HttpStatus's
/// Source - https://tools.ietf.org/html/rfc7234 & https://httpstatuses.com
const List<HttpStatus> _statuses = [
  // Informational 1x
  HttpStatus(100, 'Continue', 'Client should continue with request.'),
  HttpStatus(101, 'Switching Protocols', 'Server is switching protocols.'),
  HttpStatus(
      102, 'Processing', 'Server has received and is processing the request.'),
  // Success 2xx
  HttpStatus(200, 'Ok', 'The request was fulfilled.'),
  HttpStatus(201, 'Created',
      'The request has been fulfilled and has resulted in one or more new resources being created.'),
  HttpStatus(202, 'Accepted',
      'The request has been accepted for processing, but the processing has not been completed. The request might or might not eventually be acted upon, as it might be disallowed when processing actually takes place.'),
  HttpStatus(203, 'Non-Authoritative Information',
      'The request was successful but the enclosed payload has been modified from that of the origin server\'s 200 OK response by a transforming proxy.'),
  HttpStatus(204, 'No Content',
      'The server has successfully fulfilled the request and that there is no additional content to send in the response payload body.'),
  HttpStatus(205, 'Reset Content',
      'The server has fulfilled the request and desires that the user agent reset the document view, which caused the request to be sent, to its original state as received from the origin server.'),
  HttpStatus(206, 'Partial Content',
      'The server is successfully fulfilling a range request for the target resource by transferring one or more parts of the selected representation that correspond to the satisfiable ranges found in the request\'s Range header field.'),
  HttpStatus(207, 'Multi-Status',
      'A Multi-Status response conveys information about multiple resources in situations where multiple status codes might be appropriate.'),
  HttpStatus(208, 'Already Reported',
      'Used inside a DAV: propstat response element to avoid enumerating the internal members of multiple bindings to the same collection repeatedly.'),
  HttpStatus(226, 'Im Used',
      'The server has fulfilled a GET request for the resource, and the response is a representation of the result of one or more instance-manipulations applied to the current instance.'),
  // Redirection 3xx
  HttpStatus(300, 'Multiple Choices',
      'The target resource has more than one representation, each with its own more specific identifier, and information about the alternatives is being provided so that the user (or user agent) can select a preferred representation by redirecting its request to one or more of those identifiers.'),
  HttpStatus(301, 'Moved Permenently',
      'The target resource has been assigned a new permanent URI and any future references to this resource ought to use one of the enclosed URIs.'),
  HttpStatus(302, 'Found',
      'The target resource resides temporarily under a different URI. Since the redirection might be altered on occasion, the client ought to continue to use the effective request URI for future requests.'),
  HttpStatus(303, 'See Other',
      'The server is redirecting the user agent to a different resource, as indicated by a URI in the Location header field, which is intended to provide an indirect response to the original request.'),
  HttpStatus(304, 'Not Modified',
      'A conditional GET or HEAD request has been received and would have resulted in a 200 OK response if it were not for the fact that the condition evaluated to false.'),
  HttpStatus(305, 'Use Proxy',
      'Defined in a previous version of this specification and is now deprecated, due to security concerns regarding in-band configuration of a proxy.'),
  HttpStatus(307, 'Temporary Redirect',
      'The target resource resides temporarily under a different URI and the user agent MUST NOT change the request method if it performs an automatic redirection to that URI.'),
  HttpStatus(308, 'Permanent Redirect',
      'The target resource has been assigned a new permanent URI and any future references to this resource ought to use one of the enclosed URIs.'),
  // Client Error 4xx
  HttpStatus(400, 'Bad Request',
      'The server cannot or will not process the request due to something that is perceived to be a client error (e.g., malformed request syntax, invalid request message framing, or deceptive request routing).'),
  HttpStatus(401, 'Unauthorized',
      'The request has not been applied because it lacks valid authentication credentials for the target resource.'),
  HttpStatus(402, 'Payment Required', 'Reserved for future use.'),
  HttpStatus(403, 'Forbidden',
      'The server understood the request but refuses to authorize it.'),
  HttpStatus(404, 'Not Found',
      'The origin server did not find a current representation for the target resource or is not willing to disclose that one exists.'),
  HttpStatus(405, 'Method Not Allowed',
      'The method received in the request-line is known by the origin server but not supported by the target resource.'),
  HttpStatus(406, 'Not Acceptable',
      'The target resource does not have a current representation that would be acceptable to the user agent, according to the proactive negotiation header fields received in the request1, and the server is unwilling to supply a default representation.'),
  HttpStatus(407, 'Proxy Authentication Required',
      'Similar to 401 Unauthorized, but it indicates that the client needs to authenticate itself in order to use a proxy.'),
  HttpStatus(408, 'Request Timeout',
      'The server did not receive a complete request message within the time that it was prepared to wait.'),
  HttpStatus(409, 'Conflict',
      'The request could not be completed due to a conflict with the current state of the target resource. This code is used in situations where the user might be able to resolve the conflict and resubmit the request.'),
  HttpStatus(410, 'Gone',
      'The target resource is no longer available at the origin server and that this condition is likely to be permanent.'),
  HttpStatus(411, 'Length Required',
      'The server refuses to accept the request without a defined Content-Length.'),
  HttpStatus(412, 'Precondition Failed',
      'One or more conditions given in the request header fields evaluated to false when tested on the server.'),
  HttpStatus(413, 'Payload Too Large',
      'The server is refusing to process a request because the request payload is larger than the server is willing or able to process.'),
  HttpStatus(414, 'Request-Uri Too Log',
      'The server is refusing to service the request because the request-target is longer than the server is willing to interpret.'),
  HttpStatus(415, 'Unsupported Media Type',
      'The origin server is refusing to service the request because the payload is in a format not supported by this method on the target resource.'),
  HttpStatus(416, 'Request Range Not Satisfiable',
      'None of the ranges in the request\'s Range header field1 overlap the current extent of the selected resource or that the set of ranges requested has been rejected due to invalid ranges or an excessive request of small or overlapping ranges.'),
  HttpStatus(417, 'Expectation Failed',
      'The expectation given in the request\'s Expect header field could not be met by at least one of the inbound servers.'),
  HttpStatus(418, 'I\'m A Teapot',
      'The HTTP 418 I\'m a teapot client error response code indicates that the server refuses to brew coffee because it is, permanently, a teapot. A combined coffee/tea pot that is temporarily out of coffee should instead return 503. This error is a reference to Hyper Text Coffee Pot Control Protocol defined in April Fools\' jokes in 1998 and 2014.'),
  HttpStatus(421, 'Misdirected Request',
      'The request was directed at a server that is not able to produce a response. This can be sent by a server that is not configured to produce responses for the combination of scheme and authority that are included in the request URI.'),
  HttpStatus(422, 'Unprocessable Entity',
      'The server understands the content type of the request entity (hence a 415 Unsupported Media Type status code is inappropriate), and the syntax of the request entity is correct (thus a 400 Bad Request status code is inappropriate) but was unable to process the contained instructions.'),
  HttpStatus(423, 'Locked',
      'The source or destination resource of a method is locked.'),
  HttpStatus(424, 'Failed Dependency',
      'The method could not be performed on the resource because the requested action depended on another action and that action failed.'),
  HttpStatus(426, 'Upgrade Required',
      'The server refuses to perform the request using the current protocol but might be willing to do so after the client upgrades to a different protocol.'),
  HttpStatus(428, 'Precondition Required',
      'The origin server requires the request to be conditional.'),
  HttpStatus(429, 'Too Many Requests',
      'The user has sent too many requests in a given amount of time ("rate limiting").'),
  HttpStatus(431, 'Request Header Fields Too Large',
      'The server is unwilling to process the request because its header fields are too large. The request MAY be resubmitted after reducing the size of the request header fields.'),
  HttpStatus(444, 'Connection Closed Without Response',
      'A non-standard status code used to instruct nginx to close the connection without sending a response to the client, most commonly used to deny malicious or malformed requests.'),
  HttpStatus(451, 'Unavailable For Legal Reasons',
      'The server is denying access to the resource as a consequence of a legal demand.'),
  HttpStatus(499, 'Client Closed Request',
      'A non-standard status code introduced by nginx for the case when a client closes the connection while nginx is processing the request.'),
  // Server Error 5xx
  HttpStatus(500, 'Internal Server Error',
      'The server encountered an unexpected condition that prevented it from fulfilling the request.'),
  HttpStatus(501, 'Not Implemented',
      'The server does not support the functionality required to fulfill the request.'),
  HttpStatus(502, 'Bad Gateway',
      'The server, while acting as a gateway or proxy, received an invalid response from an inbound server it accessed while attempting to fulfill the request.'),
  HttpStatus(503, 'Service Unavailable',
      'The server is currently unable to handle the request due to a temporary overload or scheduled maintenance, which will likely be alleviated after some delay.'),
  HttpStatus(504, 'Gateway Timeout',
      'The server, while acting as a gateway or proxy, did not receive a timely response from an upstream server it needed to access in order to complete the request.'),
  HttpStatus(505, 'Http Version Not Supported',
      'The server does not support, or refuses to support, the major version of HTTP that was used in the request message.'),
  HttpStatus(506, 'Varient Also Negotiates',
      'The server has an internal configuration error: the chosen variant resource is configured to engage in transparent content negotiation itself, and is therefore not a proper end point in the negotiation process.'),
  HttpStatus(507, 'Insufficient Storage',
      'The method could not be performed on the resource because the server is unable to store the representation needed to successfully complete the request.'),
  HttpStatus(508, 'Loop Detected',
      'The server terminated an operation because it encountered an infinite loop while processing a request with "Depth: infinity". This status indicates that the entire operation failed.'),
  HttpStatus(510, 'Not Extended',
      'The policy for accessing the resource has not been met in the request. The server should send back all the information necessary for the client to issue an extended request'),
  HttpStatus(511, 'Network Authentication Required',
      'The client needs to authenticate to gain network access.'),
  HttpStatus(599, 'Network Connect Timeout Error',
      'This status code is not specified in any RFCs, but is used by some HTTP proxies to signal a network connect timeout behind the proxy to a client in front of the proxy.'),
];


/// Get HttpStatus By code
HttpStatus getStatus(int code) {
  HttpStatus _status;
  // linear search
  for (var i = 0; i < _statuses.length; i++) {
    if (_statuses[i].code == code) {
      _status = _statuses[i];
      break;
    }
  }

  if(_status == null) throw ('Invalid status code $code');
  return _status;
}
